@@ -1,10 +1,10 @@
-###############################################################################
+##############################################################################
 # correlationplus - A Python package to calculate, visualize and analyze      #
-#                    dynamical correlations maps of proteins.                 #
+#                   correlation maps of proteins.                             #
 # Authors: Mustafa Tekpinar                                                   #
-# Copyright Mustafa Tekpinar 2017-2018                                        #
-# Copyright CNRS-UMR3528, 2019                                                #
-# Copyright Institut Pasteur Paris, 2020                                      #
+# Copyright (C) Mustafa Tekpinar, 2017-2018                                   #
+# Copyright (C) CNRS-UMR3528, 2019                                            #
+# Copyright (C) Institut Pasteur Paris, 2020-2021                             #
 #                                                                             #
 # This file is part of correlationplus.                                       #
 #                                                                             #
@@ -23,20 +23,21 @@
 ###############################################################################
 
 import sys
-
+from correlationplus import __version__ as cp_vers
 # cannot use relative imports
 # as it does not work in tests/run_test
 # ImportError: attempted relative import with no known parent package
 from correlationplus.scripts.calculate import calculateApp
-from correlationplus.scripts.visualizemap import visualizemapApp
+from correlationplus.scripts.visualize import visualizemapApp
 from correlationplus.scripts.diffMap import diffMapApp
-from correlationplus.scripts.centralAnalysis import centralityAnalysisApp
+from correlationplus.scripts.analyze import centralityAnalysisApp
+from correlationplus.scripts.paths import pathAnalysisApp
 
 #TODO:
 # There are a bunch of things one can add to this script:
 # 1-Plot nDCC maps or normalized linear mutual information maps!: Done!
 # 2-Project (high) correlations onto PDB structure.
-#   a) as a Pymol script output
+#   a) as a Pymol script output: Done
 #   b) as a VMD script output: Done!
 # 3-Project secondary structures on x and y axes of a correlation map.
 # 4-Difference maps: Done!
@@ -45,6 +46,14 @@ from correlationplus.scripts.centralAnalysis import centralityAnalysisApp
 # 7-Filter correlations for residues that are very close.: Done!
 # 8-Add centrality calculations: Done
 # 9-Add centrality visualizations: Done
+# 10-Time lagged correlation analysis for MD trajectories:
+# 11-Time shifted correlation analysis. For example, shift 100 frames each time
+#    to do centrality or community analysis.
+# 12-Better pruning functions to clean correlation maps.
+# 13-A new utilities module to add, subtract, average, triangulate the correlation 
+#    maps. Some of the currently available functionalities can be moved there.
+# 14-A better way to run tests more systematically.
+# 15-Move to argparse instead of getopt.  
 
 
 def usage_main():
@@ -56,15 +65,16 @@ Example usage:
 
 correlationplus -h
 
-CorrelationPlus contains four analysis apps:
+CorrelationPlus contains five apps:
  - calculate
- - visualizemap
+ - visualize
+ - analyze
+ - paths
  - diffMap
- - centralityAnalysis
 
 You can get more information about each individual app as follows:
 
-correlationplus centralityAnalysis -h
+correlationplus analyze -h
 """)
 
 
@@ -76,25 +86,29 @@ def main():
 |                                                                            |
 |        A Python package to calculate, visualize and analyze protein        |
 |                           correlation maps.                                |
-|                   Copyright Mustafa Tekpinar 2017-2018                     |
-|                   Copyright CNRS-UMR3528, 2019                             |
-|                   Copyright Institut Pasteur Paris, 2020                   |
-|                         Author Mustafa Tekpinar                            |
+|               Copyright (C) Mustafa Tekpinar, 2017-2018                    |
+|                   Copyright (C) CNRS-UMR3528, 2019                         |
+|             Copyright (C) Institut Pasteur Paris, 2020-2021                |
+|                         Author: Mustafa Tekpinar                           |
 |                       Email: tekpinar@buffalo.edu                          |
-|                          Licence: GNU Lesser GPL                           |
+|                           Licence: GNU LGPL V3                             |
+|     Please cite us: https://pubs.acs.org/doi/10.1021/acs.jcim.1c00742      |
+|                              Version: {0}                                |
 |--------------------------------------------------------------------------- |
 
-""")
+""".format(cp_vers))
 
     if len(sys.argv) > 1:
         if sys.argv[1] == "calculate":
             calculateApp()
-        elif sys.argv[1] == "visualizemap":
+        elif sys.argv[1] == "visualize":
             visualizemapApp()
+        elif sys.argv[1] == "analyze":
+            centralityAnalysisApp()
+        elif sys.argv[1] == "paths":
+            pathAnalysisApp()
         elif sys.argv[1] == "diffMap":
             diffMapApp()
-        elif sys.argv[1] == "centralityAnalysis":
-            centralityAnalysisApp()
         elif sys.argv[1] == "-h" or sys.argv[1] == "--help":
             usage_main()
         else:
